=== modified file 'drizzled/handler.cc'
--- drizzled/handler.cc	2008-12-16 23:22:36 +0000
+++ drizzled/handler.cc	2008-12-18 23:54:47 +0000
@@ -39,6 +39,7 @@
 #include <drizzled/lock.h>
 #include <drizzled/item/int.h>
 #include <drizzled/item/empty_string.h>
+#include <drizzled/unireg.h> // for mysql_frm_type
 
 #if defined(CMATH_NAMESPACE)
 using namespace CMATH_NAMESPACE;
@@ -3002,6 +3003,7 @@
   const char *db;
   const char *name;
   int err;
+  handlerton* hton;
 };
 
 static bool table_exists_in_engine_handlerton(Session *session, plugin_ref plugin,
@@ -3017,14 +3019,19 @@
 
   vargs->err = err;
   if (vargs->err == HA_ERR_TABLE_EXIST)
+  {
+    vargs->hton= hton;
     return true;
+  }
 
   return false;
 }
 
-int ha_table_exists_in_engine(Session* session, const char* db, const char* name)
+int ha_table_exists_in_engine(Session* session,
+                              const char* db, const char* name,
+                              handlerton **hton)
 {
-  st_table_exists_in_engine_args args= {db, name, HA_ERR_NO_SUCH_TABLE};
+  st_table_exists_in_engine_args args= {db, name, HA_ERR_NO_SUCH_TABLE, NULL};
   plugin_foreach(session, table_exists_in_engine_handlerton,
                  DRIZZLE_STORAGE_ENGINE_PLUGIN, &args);
 
@@ -3039,8 +3046,17 @@
       args.err= HA_ERR_TABLE_EXIST;
     else
       args.err= HA_ERR_NO_SUCH_TABLE;
+
+    enum legacy_db_type table_type;
+    if(args.err==HA_ERR_TABLE_EXIST && mysql_frm_type(path, &table_type)==0)
+    {
+      args.hton= ha_resolve_by_legacy_type(session, table_type);
+    }
   }
 
+  if(hton)
+    *hton= args.hton;
+
   return(args.err);
 }
 

=== modified file 'drizzled/handler.h'
--- drizzled/handler.h	2008-12-15 06:30:20 +0000
+++ drizzled/handler.h	2008-12-18 23:54:47 +0000
@@ -1165,7 +1165,7 @@
 
 int ha_find_files(Session *session,const char *db,const char *path,
                   const char *wild, bool dir, List<LEX_STRING>* files);
-int ha_table_exists_in_engine(Session* session, const char* db, const char* name);
+int ha_table_exists_in_engine(Session* session, const char* db, const char* name, handlerton **hton= NULL);
 
 /* key cache */
 extern "C" int ha_init_key_cache(const char *name, KEY_CACHE *key_cache);

=== modified file 'drizzled/rename.cc'
--- drizzled/rename.cc	2008-12-15 06:52:57 +0000
+++ drizzled/rename.cc	2008-12-18 23:54:47 +0000
@@ -159,9 +159,6 @@
 {
   int rc= 1;
   const char *new_alias, *old_alias;
-  /* TODO: What should this really be set to - it doesn't
-     get set anywhere before it's used? */
-  enum legacy_db_type table_type=DB_TYPE_UNKNOWN;
 
   if (lower_case_table_names == 2)
   {
@@ -174,6 +171,15 @@
     new_alias= new_table_name;
   }
 
+  handlerton *hton= NULL;
+
+  if(ha_table_exists_in_engine(session, ren_table->db, old_alias, &hton)
+     != HA_ERR_TABLE_EXIST)
+  {
+    my_error(ER_NO_SUCH_TABLE, MYF(0), ren_table->db, old_alias);
+    return(1);
+  }
+
   if (ha_table_exists_in_engine(session, new_db, new_alias)
       !=HA_ERR_NO_SUCH_TABLE)
   {
@@ -181,9 +187,9 @@
     return(1);			// This can't be skipped
   }
 
-  rc= mysql_rename_table(ha_resolve_by_legacy_type(session, table_type),
-                               ren_table->db, old_alias,
-                               new_db, new_alias, 0);
+  rc= mysql_rename_table(hton,
+                         ren_table->db, old_alias,
+                         new_db, new_alias, 0);
   if (rc && !skip_error)
     return(1);
 

=== modified file 'drizzled/unireg.cc'
--- drizzled/unireg.cc	2008-12-14 08:26:38 +0000
+++ drizzled/unireg.cc	2008-12-18 23:54:47 +0000
@@ -88,6 +88,39 @@
   return is_handled;
 }
 
+int mysql_frm_type(char *path, enum legacy_db_type *dbt)
+{
+  int file;
+  unsigned char header[10];
+  int error;
+
+  *dbt= DB_TYPE_UNKNOWN;
+
+  file= open(path, O_RDONLY);
+
+  if(file < 1)
+    return 1;
+
+  error= read(file, header, sizeof(header));
+  close(file);
+
+  if (error!=sizeof(header))
+    return 1;
+
+  /*
+    This is just a check for DB_TYPE. We'll return default unknown type
+    if the following test is true (arg #3). This should not have effect
+    on return value from this function (default FRMTYPE_TABLE)
+  */
+  if (header[0] != (unsigned char)254 || header[1] != 1 ||
+      (header[2] != FRM_VER && header[2] != FRM_VER+1 &&
+       (header[2] < FRM_VER+3 || header[2] > FRM_VER+4)))
+    return 0;
+
+  *dbt= (enum legacy_db_type) (uint) *(header + 3);
+  return 0;
+}
+
 /*
   Create a frm (table definition) file
 

=== modified file 'drizzled/unireg.h'
--- drizzled/unireg.h	2008-12-10 07:43:50 +0000
+++ drizzled/unireg.h	2008-12-18 23:54:47 +0000
@@ -160,6 +160,8 @@
 void unireg_end(void) __attribute__((noreturn));
 void unireg_abort(int exit_code) __attribute__((noreturn));
 
+int mysql_frm_type(char *path, enum legacy_db_type *dbt);
+
 bool mysql_create_frm(Session *session, const char *file_name,
                       const char *db, const char *table,
                       HA_CREATE_INFO *create_info,

=== renamed file 'tests/suite/broken/r/rename.result' => 'tests/r/rename.result'
--- tests/suite/broken/r/rename.result	2008-06-25 05:29:13 +0000
+++ tests/r/rename.result	2008-12-18 23:54:47 +0000
@@ -27,7 +27,7 @@
 rename table t3 to t1, t2 to t3, t1 to t2, t4 to t1;
 Got one of the listed errors
 rename table t3 to t4, t5 to t3, t1 to t2, t4 to t1;
-Got one of the listed errors
+ERROR 42S02: Table 'test.t5' doesn't exist
 select * from t1;
 1	table 1
 1	table 1
@@ -40,29 +40,3 @@
 drop table if exists t1,t2,t3,t4;
 Warnings:
 Note	1051	Unknown table 't4'
-CREATE TABLE t1 (a int);
-CREATE TABLE t3 (a int);
-FLUSH TABLES WITH READ LOCK;
-RENAME TABLE t1 TO t2, t3 to t4;
-show tables;
-Tables_in_test
-t1
-t3
-UNLOCK TABLES;
-show tables;
-Tables_in_test
-t2
-t4
-drop table t2, t4;
-End of 4.1 tests
-create table t1(f1 int);
-create view v1 as select * from t1;
-alter table v1 rename to v2;
-alter table v1 rename to v2;
-ERROR 42S02: Table 'test.v1' doesn't exist
-rename table v2 to v1;
-rename table v2 to v1;
-ERROR 42S01: Table 'v1' already exists
-drop view v1;
-drop table t1;
-End of 5.0 tests

=== added file 'tests/suite/broken/r/mysql_bug2397.result'
--- tests/suite/broken/r/mysql_bug2397.result	1970-01-01 00:00:00 +0000
+++ tests/suite/broken/r/mysql_bug2397.result	2008-12-18 23:54:47 +0000
@@ -0,0 +1,14 @@
+CREATE TABLE t1 (a int);
+CREATE TABLE t3 (a int);
+FLUSH TABLES WITH READ LOCK;
+RENAME TABLE t1 TO t2, t3 to t4;
+show tables;
+Tables_in_test
+t1
+t3
+UNLOCK TABLES;
+show tables;
+Tables_in_test
+t2
+t4
+drop table t2, t4;

=== added file 'tests/suite/broken/t/mysql_bug2397.test'
--- tests/suite/broken/t/mysql_bug2397.test	1970-01-01 00:00:00 +0000
+++ tests/suite/broken/t/mysql_bug2397.test	2008-12-18 23:54:47 +0000
@@ -0,0 +1,34 @@
+#
+# Bug #2397 RENAME TABLES is not blocked by
+# FLUSH TABLES WITH READ LOCK
+#
+
+connect (con1,localhost,root,,);
+connect (con2,localhost,root,,);
+
+connection con1;
+CREATE TABLE t1 (a int);
+CREATE TABLE t3 (a int);
+connection con2;
+FLUSH TABLES WITH READ LOCK;
+connection con1;
+send RENAME TABLE t1 TO t2, t3 to t4;
+connection con2;
+show tables;
+UNLOCK TABLES;
+connection con1;
+reap;
+connection con2;
+
+# Wait for the the tables to be renamed
+# i.e the query below succeds
+let $query= select * from t2, t4;
+source include/wait_for_query_to_suceed.inc;
+
+show tables;
+
+drop table t2, t4;
+
+disconnect con2;
+disconnect con1;
+connection default;

=== renamed file 'tests/suite/broken/t/rename.test' => 'tests/t/rename.test'
--- tests/suite/broken/t/rename.test	2008-06-25 05:29:13 +0000
+++ tests/t/rename.test	2008-12-18 23:54:47 +0000
@@ -28,7 +28,7 @@
 show tables like "t_";
 --error 1050,1050
 rename table t3 to t1, t2 to t3, t1 to t2, t4 to t1;
---error 1017,1017
+--error ER_NO_SUCH_TABLE
 rename table t3 to t4, t5 to t3, t1 to t2, t4 to t1;
 
 select * from t1;
@@ -36,60 +36,4 @@
 select * from t3;
 
 # This should give a warning for t4
-drop table if exists t1,t2,t3,t4;
-
-#
-# Bug #2397 RENAME TABLES is not blocked by
-# FLUSH TABLES WITH READ LOCK
-#
-
-connect (con1,localhost,root,,);
-connect (con2,localhost,root,,);
-
-connection con1;
-CREATE TABLE t1 (a int);
-CREATE TABLE t3 (a int);
-connection con2;
-FLUSH TABLES WITH READ LOCK;
-connection con1;
-send RENAME TABLE t1 TO t2, t3 to t4;
-connection con2;
-show tables;
-UNLOCK TABLES;
-connection con1;
-reap;
-connection con2;
-
-# Wait for the the tables to be renamed
-# i.e the query below succeds
-let $query= select * from t2, t4;
-source include/wait_for_query_to_suceed.inc;
-
-show tables;
-
-drop table t2, t4;
-
-disconnect con2;
-disconnect con1;
-connection default;
-
-
---echo End of 4.1 tests
-
-
-#
-# Bug#14959: ALTER TABLE isn't able to rename a view
-#
-create table t1(f1 int);
-create view v1 as select * from t1;
-alter table v1 rename to v2;
---error 1146
-alter table v1 rename to v2;
-rename table v2 to v1;
---error 1050
-rename table v2 to v1;
-drop view v1;
-drop table t1;
-
-
---echo End of 5.0 tests
+drop table if exists t1,t2,t3,t4;
\ No newline at end of file

